@@ -688,6 +688,9 @@ static bool isErrorExpr(const char* output)
 static bool parseErrorMessage(const char*& output,
 			      ExprValue*& variable, bool wantErrorValue)
 {
+    while (isspace(*output))
+        output++;
+
     // skip warnings
     while (strncmp(output, "warning:", 8) == 0)
     {
@@ -696,6 +699,8 @@ static bool parseErrorMessage(const char*& output,
 	    output += strlen(output);
 	else
 	    output = end+1;
+        while (isspace(*output))
+            output++;
     }
 
     if (isErrorExpr(output))
@@ -1099,6 +1104,23 @@ moreStrings:
 	    }
 	}
     }
+    /*
+     * There's a bug in gdb where it prints the beginning of the string
+     * continuation and the comma-blank in the wrong order if the new string
+     * begins with an incomplete multi-byte character. For now, let's check
+     * for this in a very narrow condition, particularly, where the next
+     * character is given in octal notation. Example:
+     *     'a' <repeats 20 times>"\240, b"
+     */
+    if (*p == '"' && p[1] == '\\' && isdigit(p[2])) {
+	int i = 3;
+	while (isdigit(p[i]))
+	    ++i;
+	if (p[i] == ',' && p[i+1] == ' ') {
+	    // just treat everything beginning at the dquote as string
+	    goto moreStrings;
+	}
+    }
     /* very long strings are followed by `...' */
     if (*p == '.' && p[1] == '.' && p[2] == '.') {
 	p += 3;
@@ -1800,24 +1822,32 @@ bool GdbDriver::parseBreakList(const char* output, std::list<Breakpoint>& brks)
 	// get Num
 	bp.id = strtol(p, &dummy, 10);	/* don't care about overflows */
 	p = dummy;
-	// get Type
-	while (isspace(*p))
-	    p++;
-	if (strncmp(p, "breakpoint", 10) == 0) {
-	    p += 10;
-	} else if (strncmp(p, "hw watchpoint", 13) == 0) {
-	    bp.type = Breakpoint::watchpoint;
-	    p += 13;
-	} else if (strncmp(p, "watchpoint", 10) == 0) {
-	    bp.type = Breakpoint::watchpoint;
-	    p += 10;
+	// check for continued <MULTIPLE> breakpoint
+	if (*p == '.' && isdigit(p[1]))
+	{
+	    // continuation: skip type and disposition
+	}
+	else
+	{
+	    // get Type
+	    while (isspace(*p))
+		p++;
+	    if (strncmp(p, "breakpoint", 10) == 0) {
+		p += 10;
+	    } else if (strncmp(p, "hw watchpoint", 13) == 0) {
+		bp.type = Breakpoint::watchpoint;
+		p += 13;
+	    } else if (strncmp(p, "watchpoint", 10) == 0) {
+		bp.type = Breakpoint::watchpoint;
+		p += 10;
+	    }
+	    while (isspace(*p))
+		p++;
+	    if (*p == '\0')
+		break;
+	    // get Disp
+	    bp.temporary = *p++ == 'd';
 	}
-	while (isspace(*p))
-	    p++;
-	if (*p == '\0')
-	    break;
-	// get Disp
-	bp.temporary = *p++ == 'd';
 	while (*p != '\0' && !isspace(*p))	/* "keep" or "del" */
 	    p++;
 	while (isspace(*p))
@@ -1851,7 +1881,9 @@ bool GdbDriver::parseBreakList(const char* output, std::list<Breakpoint>& brks)
 	    bp.location = p;
 	    p += bp.location.length();
 	} else {
-	    bp.location = QString::fromLatin1(p, end-p).trimmed();
+	    // location of a <MULTIPLE> filled in from subsequent breakpoints
+	    if (strncmp(p, "<MULTIPLE>", 10) != 0)
+		bp.location = QString::fromLatin1(p, end-p).trimmed();
 	    p = end+1;			/* skip over \n */
 	}
 
@@ -1889,7 +1921,20 @@ bool GdbDriver::parseBreakList(const char* output, std::list<Breakpoint>& brks)
 	    if (*p != '\0')
 		p++;			/* skip '\n' */
 	}
-	brks.push_back(bp);
+
+	if (brks.empty() || brks.back().id != bp.id) {
+	    brks.push_back(bp);
+	} else {
+	    // this is a continuation; fill in location if not yet set
+	    // otherwise, drop this breakpoint
+	    Breakpoint& mbp = brks.back();
+	    if (mbp.location.isEmpty() && !bp.location.isEmpty()) {
+		mbp.location = bp.location;
+		mbp.address = bp.address;
+	    } else if (mbp.address.isEmpty() && !bp.address.isEmpty()) {
+		mbp.address = bp.address;
+	    }
+	}
     }
     return true;
 }
@@ -1951,6 +1996,7 @@ std::list<ThreadInfo> GdbDriver::parseThreadList(const char* output)
 		// syntax error; bail out
 		return threads;
 	    }
+	    end += 2;
 	} else {
 	    // In the new format lies crazyness: there is no definitive
 	    // end marker. At best we can guess when the SYSTAG ends.
@@ -1984,8 +2030,8 @@ std::list<ThreadInfo> GdbDriver::parseThreadList(const char* output)
 		    ++end;
 	    }
 	}
-	thr.threadName = QString::fromLatin1(p, end-p);
-	p = end+2;
+	thr.threadName = QString::fromLatin1(p, end-p).trimmed();
+	p = end;
 
 	/*
 	 * Now follows a standard stack frame. Sometimes, however, gdb
@@ -2012,25 +2058,26 @@ static bool parseNewWatchpoint(const char* o, int& id,
 bool GdbDriver::parseBreakpoint(const char* output, int& id,
 				QString& file, int& lineNo, QString& address)
 {
-    const char* o = output;
     // skip lines of that begin with "(Cannot find"
-    while (strncmp(o, "(Cannot find", 12) == 0) {
-	o = strchr(o, '\n');
-	if (o == 0)
+    while (strncmp(output, "(Cannot find", 12) == 0 ||
+	   strncmp(output, "Note: breakpoint", 16) == 0)
+    {
+	output = strchr(output, '\n');
+	if (output == 0)
 	    return false;
-	o++;				/* skip newline */
+	output++;			/* skip newline */
     }
 
-    if (strncmp(o, "Breakpoint ", 11) == 0) {
+    if (strncmp(output, "Breakpoint ", 11) == 0) {
 	output += 11;			/* skip "Breakpoint " */
 	return ::parseNewBreakpoint(output, id, file, lineNo, address);
-    } else if (strncmp(o, "Temporary breakpoint ", 21) == 0) {
+    } else if (strncmp(output, "Temporary breakpoint ", 21) == 0) {
 	output += 21;
 	return ::parseNewBreakpoint(output, id, file, lineNo, address);
-    } else if (strncmp(o, "Hardware watchpoint ", 20) == 0) {
+    } else if (strncmp(output, "Hardware watchpoint ", 20) == 0) {
 	output += 20;
 	return ::parseNewWatchpoint(output, id, address);
-    } else if (strncmp(o, "Watchpoint ", 11) == 0) {
+    } else if (strncmp(output, "Watchpoint ", 11) == 0) {
 	output += 11;
 	return ::parseNewWatchpoint(output, id, address);
     }
@@ -2054,17 +2101,38 @@ static bool parseNewBreakpoint(const char* o, int& id,
 	    ++p;
 	address = QString::fromLatin1(start, p-start);
     }
-    
-    // file name
+
+    /*
+     * Mostly, GDB responds with this syntax:
+     *
+     * Breakpoint 1 at 0x400b94: file multibrkpt.cpp, line 9. (2 locations)
+     *
+     * but sometimes it uses this syntax:
+     *
+     * Breakpoint 4 at 0x804f158: lotto739.cpp:95. (3 locations)
+     */
+    char* fileEnd, *numStart = 0;
     char* fileStart = strstr(p, "file ");
-    if (fileStart == 0)
-	return !address.isEmpty();     /* parse error only if there's no address */
-    fileStart += 5;
-    
-    // line number
-    char* numStart = strstr(fileStart, ", line ");
-    QString fileName = QString::fromLatin1(fileStart, numStart-fileStart);
-    numStart += 7;
+    if (fileStart != 0)
+    {
+	fileStart += 5;
+	fileEnd = strstr(fileStart, ", line ");
+	if (fileEnd != 0)
+	    numStart = fileEnd + 7;
+    }
+    if (numStart == 0 && p[0] == ':' && p[1] == ' ')
+    {
+	fileStart = p+2;
+	while (isspace(*fileStart))
+	    ++fileStart;
+	fileEnd = strchr(fileStart, ':');
+	if (fileEnd != 0)
+	    numStart = fileEnd + 1;
+    }
+    if (numStart == 0)
+	return !address.isEmpty();	/* parse error only if there's no address */
+
+    QString fileName = QString::fromLatin1(fileStart, fileEnd-fileStart);
     int line = strtoul(numStart, &p, 10);
     if (numStart == p)
 	return false;
@@ -2165,8 +2233,10 @@ bool GdbDriver::parseChangeExecutable(const char* output, QString& message)
      * Lines starting with the following do not indicate errors:
      *     Using host libthread_db
      *     (no debugging symbols found)
+     *     Reading symbols from
      */
-    while (strncmp(output, "Using host libthread_db", 23) == 0 ||
+    while (strncmp(output, "Reading symbols from", 20) == 0 ||
+	   strncmp(output, "Using host libthread_db", 23) == 0 ||
 	   strncmp(output, "(no debugging symbols found)", 28) == 0)
     {
 	// this line is good, go to the next one
